//***************************************************************************
//                          main.cpp  -  description
//                             -------------------
//  begin            : Sa. Feb 6 10:13:45 2016
//  generated by     : pvdevelop (C) Lehrig Software Engineering
//  email            : lehrig@t-online.de
//
//  You may run this example pvserver on a Raspberry Pi
//  It controls a MCP23017 I2C Port Expander via the I2C bus 
//  sudo apt-get install i2c-tools
//  The I2C bus is handled within main.cpp in the thread function "thread_main()"
//  The following global variables store the GPIO status on the port exander
//  int gpioa;
//  int gpiob;
//***************************************************************************
#include "pvapp.h"
// todo: comment me out. you can insert these objects as extern in your masks.
// Change "modbus"(Change if applicable) and "modbusdaemon_" with the "name" of your file "name.mkmodbus".
//rlModbusClient     modbus(modbusdaemon_MAILBOX,modbusdaemon_SHARED_MEMORY,modbusdaemon_SHARED_MEMORY_SIZE);
//rlSiemensTCPClient siemensTCP(siemensdaemon_MAILBOX,siemensdaemon_SHARED_MEMORY,siemensdaemon_SHARED_MEMORY_SIZE);
//rlPPIClient        ppi(ppidaemon_MAILBOX,ppidaemon_SHARED_MEMORY,ppidaemon_SHARED_MEMORY_SIZE);

rlThread mythread;                          // define a thread instance
const static unsigned char OP       = 0x20; // device opcode (port exander on A0=0 A1=0 A2=0)
                                            // for bank=0
const static unsigned char IODIRA   = 0x00;
const static unsigned char IPOLA    = 0x02;
const static unsigned char GPINTENA = 0x04;
const static unsigned char GPPUA    = 0x0C;
const static unsigned char GPIOA    = 0x12;
const static unsigned char OLATA    = 0x14;
const static unsigned char IODIRB   = 0x01;
const static unsigned char IPOLB    = 0x03;
const static unsigned char GPINTENB = 0x05;
const static unsigned char GPPUB    = 0x0D;
const static unsigned char GPIOB    = 0x13;
const static unsigned char OLATB    = 0x15;
// global data that can be used extern in masks
int gpioa = 0;
int gpiob = 0;

// Suche nach I2C-Adressen
void scan_i2c_bus(int device)
{
  int port, res;
  // Adressbereich 7 Bit
  for(port=0; port<127; port++)
  {
    if(ioctl(device, I2C_SLAVE, port) < 0)
    {
      perror("ioctl() I2C_SLAVE failed\n");
    }
    else
    {
      // kann gelesen werden?
      res = i2c_smbus_read_byte(device);
      if(res >= 0)
      {
        printf("i2c chip found at: %x, val = %d\n", port, res);
      }
    }
  }
}
  
void *thread_main(void *arg)
{
  printf("i2ctest starting\n");
  int file;
  unsigned long funcs;
  int old_gpioa = -1;
  int old_gpiob = -1;

  if((file=open("/dev/i2c-1", O_RDWR)) < 0)
  {
    fprintf(stderr,"failed to open the i2c bus\n");
    exit(1);
  }
  scan_i2c_bus(file);

  if(ioctl(file, I2C_SLAVE, 0x20) < 0) // connect to slave 0x20
  {
    fprintf(stderr,"failed to acquire bus access and/or talk to slave.\n");
    return NULL;
  }
  if(ioctl(file,I2C_FUNCS,&funcs) < 0) // test if I2C-functions are available 
  {
    fprintf(stderr,"ioctl() I2C_FUNCS failed");
    return NULL;
  }

  // set direction to output
  i2c_smbus_write_byte_data(file,IODIRA,0x7f); // set some pins to output 
  i2c_smbus_write_byte_data(file,IODIRB,0xfe); // set some pins to output

  while(1)
  {
    if(1)
    {
      i2c_smbus_write_byte_data(file,GPIOA,0x00); // set all output A 
      i2c_smbus_write_byte_data(file,GPIOB,0x00); // set all output B 
      gpioa = old_gpioa = i2c_smbus_read_byte_data(file,GPIOA);
      printf("old_gpioa=%d\n", old_gpioa);
      gpiob = old_gpiob = i2c_smbus_read_byte_data(file,GPIOB);
      printf("old_gpiob=%d\n", old_gpiob);
      rlsleep(1000);

      i2c_smbus_write_byte_data(file,GPIOA,0xff); // set all output A 
      i2c_smbus_write_byte_data(file,GPIOB,0xff); // set all output B 
      gpioa = old_gpioa = i2c_smbus_read_byte_data(file,GPIOA);
      printf("old_gpioa=%d\n", old_gpioa);
      gpiob = old_gpiob = i2c_smbus_read_byte_data(file,GPIOB);
      printf("old_gpiob=%d\n", old_gpiob);
      rlsleep(1000);
    }
    else
    {
      if(old_gpioa != gpioa)
      {
        i2c_smbus_write_byte_data(file,GPIOA,gpioa); // set output A
        old_gpioa = i2c_smbus_read_byte_data(file,GPIOA);
        printf("gpioa=%d\n", old_gpioa);
      }
      if(old_gpiob != gpiob)
      {
        old_gpiob = gpiob;
        i2c_smbus_write_byte_data(file,GPIOA,gpiob); // set output A
        old_gpiob = i2c_smbus_read_byte_data(file,GPIOB);
        printf("gpiob=%d\n", old_gpiob);
      }
      rlsleep(100);
    }
  }

  close(file);
  printf("i2ctest end\n");
  return arg;
}

int pvMain(PARAM *p)
{
int ret;

  pvSendVersion(p);
  pvSetCaption(p,"pvs");
  pvResize(p,0,1280,1024);
  //pvScreenHint(p,1024,768); // this may be used to automatically set the zoomfactor
  ret = 1;
  pvGetInitialMask(p);
  if(strcmp(p->initial_mask,"mask1") == 0) ret = 1;

  while(1)
  {
    if(trace) printf("show_mask%d\n", ret);
    switch(ret)
    {
      case 1:
        pvStatusMessage(p,-1,-1,-1,"mask1");
        ret = show_mask1(p);
        break;
      default:
        return 0;
    }
  }
}

#ifdef USE_INETD
int main(int ac, char **av)
{
PARAM p;

  pvInit(ac,av,&p);
  /* here you may interpret ac,av and set p->user to your data */
  pvMain(&p);
  return 0;
}
#else  // multi threaded server
int main(int ac, char **av)
{
PARAM p;
int   s;

  mythread.create(thread_main, NULL); // start i2c thread 
  pvInit(ac,av,&p);
  /* here you may interpret ac,av and set p->user to your data */
  while(1)
  {
    s = pvAccept(&p);
    if(s != -1) pvCreateThread(&p,s);
  }
  return 0;
}

#endif
